// SPDX-FileCopyrightText: Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
// SPDX-License-Identifier: BSD-3-Clause
#include "vtkCellGridShaderFragment.h"

const char *vtkCellGridShaderFragment =
"//VTK::System::Dec\n"
"// SPDX-FileCopyrightText: Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen\n"
"// SPDX-License-Identifier: BSD-3-Clause\n"
"//VTK::Camera::Dec\n"
"\n"
"//VTK::Output::Dec\n"
"\n"
"// Position of vertex in view coordinates.\n"
"//VTK::PositionVC::Dec\n"
"\n"
"// The normal of the output primitive in view coordinates.\n"
"//VTK::Normal::Dec\n"
"\n"
"// Depth Peeling Support\n"
"//VTK::DepthPeeling::Dec\n"
"\n"
"// handle coincident offsets\n"
"//VTK::Coincident::Dec\n"
"\n"
"// Value raster\n"
"//VTK::ValuePass::Dec\n"
"\n"
"// Lights\n"
"//VTK::Light::Dec\n"
"\n"
"{commonDefs}\n"
"{cellEval}\n"
"{cellUtil}\n"
"\n"
"smooth in vec3 pcoordVSOutput;\n"
"\n"
"uniform sampler2D color_map;\n"
"uniform vec3 color_range;\n"
"uniform int color_component;\n"
"// basic material shading capabilities\n"
"uniform vec3 color_ambient;\n"
"uniform vec3 color_diffuse;\n"
"uniform vec3 color_specular;\n"
"uniform vec3 color_ambient_bf; // backface\n"
"uniform vec3 color_diffuse_bf; // backface\n"
"uniform vec3 color_specular_bf; // backface\n"
"uniform float intensity_ambient;\n"
"uniform float intensity_diffuse;\n"
"uniform float intensity_specular;\n"
"uniform float intensity_opacity;\n"
"uniform float intensity_ambient_bf; // backface\n"
"uniform float intensity_diffuse_bf; // backface\n"
"uniform float intensity_specular_bf; // backface\n"
"uniform float intensity_opacity_bf; // backface\n"
"uniform float power_specular;\n"
"uniform float power_specular_bf; // backface\n"
"uniform int enable_specular;\n"
"\n"
"flat in float shapeValuesVSOutput[{ShapeCoeffPerCell}];\n"
"flat in float colorValuesVSOutput[{ColorCoeffPerCell}];\n"
"\n"
"void main()\n"
"{{\n"
"  //VTK::PositionVC::Impl\n"
"\n"
"  // Place any calls that require uniform flow (e.g. dFdx) here.\n"
"  //VTK::UniformFlow::Impl\n"
"\n"
"  // Set gl_FragDepth here (gl_FragCoord.z by default)\n"
"  //VTK::Depth::Impl\n"
"\n"
"  // Early depth peeling abort:\n"
"  //VTK::DepthPeeling::PreColor\n"
"\n"
"  float fieldValue[{ColorNumValPP}];\n"
"  float scalar; // The non-normalized scalar value computed from a fieldValue tuple.\n"
"  vec2 texCoord; // Used for color lookup.\n"
"\n"
"  if ({HaveColors})\n"
"  {{\n"
"    // Evaluate the basis function at this fragment's parametric coords (pcoordVSOutput),\n"
"    // yielding the \\a fieldValue tuple.\n"
"    colorEvaluateAt(pcoordVSOutput, shapeValuesVSOutput, colorValuesVSOutput, fieldValue);\n"
"\n"
"    // Choose how we map the \\a fieldValue tuple into a texture-map coordinate.\n"
"    switch (color_component)\n"
"    {{\n"
"    case -2:\n"
"      // L₂ norm:\n"
"      {{\n"
"        float mag = 0.0;\n"
"        for (int cc = 0; cc < {ColorNumValPP}; ++cc)\n"
"        {{\n"
"          mag += fieldValue[cc] * fieldValue[cc];\n"
"        }}\n"
"        scalar = sqrt(mag);\n"
"      }}\n"
"    case -1:\n"
"      // L₁ norm (choose the maximum across components):\n"
"      {{\n"
"        scalar = fieldValue[0];\n"
"        for (int cc = 1; cc < {ColorNumValPP}; ++cc)\n"
"        {{\n"
"          if (fieldValue[cc] > scalar)\n"
"          {{\n"
"            scalar = fieldValue[cc];\n"
"          }}\n"
"        }}\n"
"      }}\n"
"    default:\n"
"      // Choose a single component.\n"
"      scalar = fieldValue[color_component];\n"
"    }}\n"
"    if (color_range[2] > 0.0)\n"
"    {{\n"
"      texCoord = vec2((scalar - color_range[0]) / color_range[2], 0.0);\n"
"    }}\n"
"    else\n"
"    {{\n"
"      texCoord = vec2((scalar - color_range[0]), 0.0);\n"
"    }}\n"
"    // texCoord = vec2(colorValuesVSOutput[3], 0.);\n"
"  }}\n"
"\n"
"  vec4 color;\n"
"  vec3 ambientColor;\n"
"  vec3 diffuseColor;\n"
"  vec3 specularColor;\n"
"  float opacity;\n"
"  float specularPower = 0.0f;\n"
"  if ({HaveColors})\n"
"  {{\n"
"    color = texture(color_map, texCoord);\n"
"    // color = vec4((colorValuesVSOutput[0] - color_range[0])/color_range[2], (colorValuesVSOutput[1] - color_range[0])/color_range[2], (colorValuesVSOutput[2] - color_range[0])/color_range[2], 1.0);\n"
"    // color = vec4(texCoord.s, texCoord.s, texCoord.s, 1.0);\n"
"    ambientColor = intensity_ambient * color.rgb;\n"
"    diffuseColor = intensity_diffuse * color.rgb;\n"
"    specularColor = intensity_specular * color.rgb;\n"
"    opacity = intensity_opacity * color.a;\n"
"  }}\n"
"  else\n"
"  {{\n"
"    color = vec4(1, 1, 1, 1);\n"
"    // FIXME: gl_FrontFacing cannot be relied upon for some reason.\n"
"    //        1. Dual depth peeling works and sometimes doesn't.\n"
"    //        2. Backfaces are not rendered.\n"
"    // if (gl_FrontFacing == false)\n"
"    // {{\n"
"    //   ambientColor = intensity_ambient_bf * color_ambient_bf;\n"
"    //   diffuseColor = intensity_diffuse_bf * color_diffuse_bf;\n"
"    //   specularColor = intensity_specular_bf * color_specular_bf;\n"
"    //   opacity = intensity_opacity_bf;\n"
"    //   specularPower = power_specular_bf;\n"
"    // }}\n"
"    // else\n"
"    {{\n"
"      ambientColor = intensity_ambient * color_ambient;\n"
"      diffuseColor = intensity_diffuse * color_diffuse;\n"
"      specularColor = intensity_specular * color_specular;\n"
"      opacity = intensity_opacity;\n"
"      specularPower = power_specular;\n"
"    }}\n"
"  }}\n"
"\n"
"  //VTK::Normal::Impl\n"
"\n"
"  //VTK::Light::Impl\n"
"\n"
"  // Discard fully transparent pixels\n"
"  if (gl_FragData[0].a <= 0.0) discard;\n"
"\n"
"  //VTK::DepthPeeling::Impl\n"
"}}\n"
"";
